#pragma once

#include <folly/FBVector.h>

#include <queue>
#include <utility>

namespace rtransfer {

template <typename T, typename Hash = std::hash<T>>
class BucketQueue {
public:
    BucketQueue(const std::size_t buckets)
        : used_(buckets, false)
        , array_(buckets)
    {
    }

    std::pair<std::size_t, bool> push(std::uint8_t /*priority*/, T &&val)
    {
        const auto idx = bucket(val);
        array_[idx].push(std::forward<T>(val));
        return std::make_pair(idx, !used_[idx]);
    }

    void use(std::size_t idx) { used_[idx] = true; }

    void stopUsing(std::size_t idx) { used_[idx] = false; }

    bool empty(std::size_t idx) { return array_[idx].empty(); }

    T pop(std::size_t idx)
    {
        auto val = std::move(array_[idx].front());
        array_[idx].pop();
        return val;
    }

    std::size_t bucket(const T &val) { return Hash{}(val) % array_.size(); }

private:
    folly::fbvector<bool> used_;
    folly::fbvector<std::queue<T>> array_;
};

}  // namespace rtransfer
