#pragma once

#include <folly/Function.h>
#include <folly/io/async/EventBase.h>

#include <chrono>

class PeriodicHandler : public folly::HHWheelTimer::Callback {
public:
    template <typename F>
    PeriodicHandler(
        folly::EventBase *base, std::chrono::milliseconds after, F &&callback)
        : base_{base}
        , after_{after}
        , onTimeoutExpired_{std::forward<F>(callback)}
    {
        schedule();
    }

    void timeoutExpired() noexcept override
    {
        onTimeoutExpired_();
        schedule();
    }

    void callbackCanceled() noexcept override {}

private:
    void schedule() noexcept { base_->timer().scheduleTimeout(this, after_); }

    folly::EventBase *base_;
    std::chrono::milliseconds after_;
    folly::Function<void()> onTimeoutExpired_;
};
