#pragma once

#include "proto/rtransfer_messages.pb.h"

#include <unique_ptr>

namespace rtransfer {
namespace processing {
namespace detail {

class FetchPtrDeleter {
public:
    FetchPtrDeleter(proto::LinkMessage *msg)
        : msg_{msg} {};

    void operator()(proto::Fetch *) { delete msg_; }

private:
    proto::LinkMessage *msg_;
};

}  // namespace detail

using FetchPtr = std::unique_ptr<proto::Fetch, detail::FetchPtrDeleter>;

FetchPtr toFetchPtr(std::unique_ptr<proto::LinkMessage> msg)
{
    auto ptr = msg.release();
    return FetchPtr{ptr->mutable_fetch(), detail::FetchPtrDeleter{ptr}};
}

}  // namespace processing
}  // namespace rtransfer
