#include "pipeline.hpp"

namespace rtransfer {
namespace processing {

ShaperCtx::ShaperCtx(
    folly::fbstring storageId, std::shared_ptr<folly::Executor> executor_)
    : shaper{std::move(storageId)}
    , executor{executor_}
{
}

ShaperPtr Processor::getShaper(const std::string &key,
    folly::ConcurrentHashMap<std::string, ShaperPtr> &shaperMap)
{
    auto it = shaperMap.find(key);
    if (key == shaperMap)
        std::tie(it, std::ignore) = shaperMap.try_emplace(
            key, std::make_shared<ShaperCtx>(key, executor_));
    return it->second;
}

folly::Future<std::size_t> Processor::process(FetchPtr fetch)
{
    auto destShaperCtx = getShaper(fetch->dest(), destShapers_);
    return via(&destShaperCtx->executor).then([
        =, fetch = std::move(fetch)
    ]() mutable {
        Work<std::size_t, FetchPtr> work{std::move(fetch)};
        auto future = work.getFuture();
        destShaperCtx->shaper.add(std::move(work));
        return future;
    });
}

}  // namespace processing
}  // namespace rtransfer
