#pragma once

#include "fetchPtr.hpp"
#include "shaper.hpp"

#include <folly/concurrency/ConcurrentHashMap.h>
#include <folly/executors/CPUThreadPoolExecutor.h>
#include <folly/executors/SerialExecutor.h>

#include <string>
#include <thread>

namespace rtransfer {
namespace processing {

using Pipeline = wangle::Pipeline<FetchPtr, folly::Unit>;

struct ShaperCtx {
    ShaperCtx(
        folly::fbstring storageId, std::shared_ptr<folly::Executor> executor);
    Shaper shaper;
    folly::SerialExecutor executor;
};

using ShaperMap =
    folly::ConcurrentHashMap<folly::fbstring, std::shared_ptr<ShaperCtx>>;

template <typename Ret, typename T>
class Work {
    Work(T object_)
        : object{std::move(object_)}
    {
    }

    void done(Ret ret) { promise.setValue(ret); }

    folly::Future<Ret> getFuture() { return promise.getFuture(); }

    T object;

private:
    folly::Promise<Ret> promise;
};

// template <typename T>
// class Processor {
// public:
//     folly::Future<Ret>
// }

template <typename Next>
class DestShaperHandler {
public:
    DestShaperHandler(std::unique_ptr<Next> next);

    folly::Future<std::size_t> next(FetchPtr fetch);

private:
    std::unique_ptr<Next> next_;
    ShaperMap shapers_;
};

class Processor {
public:
    folly::Future<std::size_t> process(FetchPtr fetch);

private:
    std::shared_ptr<folly::CPUThreadPoolExecutor> executor_ =
        std::make_shared<folly::CPUThreadPoolExecutor>(
            std::thread::hardware_concurrency(),
            std::make_shared<folly::NamedThreadFactory>("processor"));

    ShaperMap destShapers_;
    ShaperMap srcShapers_;
};

}  // namespace processing
}  // namespace rtransfer
