#pragma once

#include "common.hpp"
#include "handleCache.hpp"
#include "serverSideLink.hpp"
#include "storage.hpp"

#include <folly/Bits.h>
#include <folly/executors/Codel.h>
#include <folly/futures/Future.h>
#include <folly/io/IOBuf.h>
#include <folly/io/IOBufQueue.h>
#include <folly/io/async/EventBase.h>
#include <monitoring/monitoring.h>

#include <cstdint>
#include <memory>

namespace rtransfer {

class ShaperMap;

class Reader {
public:
    Reader(StoragesMap &storages, HandleCache &handleCache,
        std::shared_ptr<ServerSideLink> serverLink, folly::EventBase *base);

    folly::Future<std::size_t> read(std::uint64_t reqId,
        folly::fbstring storageId, folly::fbstring fileId, std::uint64_t offset,
        std::size_t size, std::uint8_t priority);

    folly::Future<folly::Unit> cancel(
        std::uint64_t reqId, folly::fbstring storageId);

    void setShaperMap(ShaperMap *shaperMap)
    {
        base_->runInEventBaseThread([=] { shaperMap_ = shaperMap; });
    }

private:
    folly::Future<std::size_t> doRead(std::uint64_t reqId,
        const folly::fbstring &storageId, const folly::fbstring &fileId,
        std::uint64_t offset, std::size_t size, std::uint8_t priority);

    folly::Future<folly::Unit> doCancel(
        std::uint64_t reqId, folly::StringPiece storageId);

    StoragesMap &storages_;
    HandleCache &handleCache_;
    ShaperMap *shaperMap_ = nullptr;
    std::shared_ptr<ServerSideLink> serverLink_;
    folly::EventBase *base_;
    folly::Codel transportCodel_;
};

}  // namespace rtransfer
