#pragma once

#include "common.hpp"
#include "periodicHandler.hpp"
#include "priorityQueue.hpp"
#include "reader.hpp"
#include "shaper.hpp"

#include <folly/Hash.h>
#include <folly/futures/Future.h>
#include <gflags/gflags.h>
#include <wangle/channel/Handler.h>
#include <wangle/service/Service.h>
#include <boost/multi_index/hashed_index.hpp>
#include <boost/multi_index/key_extractors.hpp>
#include <boost/multi_index_container.hpp>

#include <queue>

namespace rtransfer {

struct BySrc {
};
struct ByDst {
};
struct ByKey {
};

namespace bmi = boost::multi_index;

class ShaperMap {
public:
    ShaperMap(Reader &reader, folly::EventBase *base);

    ~ShaperMap();

    folly::Future<std::size_t> fetch(MsgPtr msg);

    folly::Future<folly::Unit> cancel(MsgPtr msg);

    void congestion(MsgPtr msg);

    void sourceCongestion(folly::fbstring storageId);

    void transportCongestion();

private:
    folly::Future<std::size_t> doFetch(MsgPtr msg);

    folly::Future<folly::Unit> doCancel(MsgPtr msg);

    void doCongestion(MsgPtr msg);

    void doSourceCongestion(folly::fbstring storageId);

    template <typename Tag>
    void doNotifyCongestion(folly::fbstring storageId);

    void trigger();

    using Map = bmi::multi_index_container<std::unique_ptr<Shaper>,
        bmi::indexed_by<bmi::hashed_non_unique<bmi::tag<BySrc>,
                            bmi::member<Shaper, folly::fbstring, &Shaper::src_>,
                            std::hash<folly::fbstring>>,
            bmi::hashed_non_unique<bmi::tag<ByDst>,
                bmi::member<Shaper, folly::fbstring, &Shaper::dst_>,
                std::hash<folly::fbstring>>,
            bmi::hashed_unique<bmi::tag<ByKey>,
                bmi::composite_key<Shaper,
                    bmi::member<Shaper, folly::fbstring, &Shaper::src_>,
                    bmi::member<Shaper, folly::fbstring, &Shaper::dst_>>,
                bmi::composite_key_hash<std::hash<folly::fbstring>,
                    std::hash<folly::fbstring>>>>>;

    Map shapers_;
    Reader &reader_;
    folly::EventBase *base_;
    std::unique_ptr<PeriodicHandler> periodicHandler_;
};

}  // namespace rtransfer
